/***************************************************************************
                          decoder_thread.h  -  description
                             -------------------
    begin                : Thu Feb 23 2009
    copyright            : (C) 2009 by Martin Witte
    email                : emw-kradio@nocabal.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KRADIO_DECODER_THREAD_H
#define KRADIO_DECODER_THREAD_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QThread>
#include <QtCore/QSemaphore>

#include <kurl.h>

#include "soundformat.h"
#include "sound_metadata.h"
#include "internetradiostation.h"

#include "databuffer.h"

extern "C" {
    #include <libavformat/avformat.h>
}

class DecoderThread : public QThread
{
public:
    DecoderThread(QObject *parent, const InternetRadioStation &rs, int max_buffers);

    virtual ~DecoderThread();

    void                  run();

    bool                  error()       const { return m_error; }
    const QString        &errorString() const { return m_errorString; }

    void                  setDone();
    bool                  isDone() const { return m_done; }

    bool                  initDone() const    { return m_decoderOpened; }

    // sound format is only valid if initDone returns true
    const SoundFormat    &soundFormat() const { return m_soundFormat; }


    int                   availableBuffers();
    DataBuffer           &getFirstBuffer();
    void                  popFirstBuffer();
    void                  flushBuffers();
protected:
    void                  pushBuffer(const DataBuffer &);


protected:
    void                  addErrorString(const QString &s);

    bool                  decoderOpened() const { return m_decoderOpened; }
    void                  openAVStream(const QString &stream);
    void                  closeAVStream();



    bool                  m_decoderOpened;
    AVFormatContext      *m_av_pFormatCtx;
    int                   m_av_audioStream;
    AVCodecContext       *m_av_aCodecCtx;
    AVCodec              *m_av_aCodec;


    QObject              *m_parent;
    InternetRadioStation  m_RadioStation;

    bool                  m_error;
    QString               m_errorString;
    bool                  m_done;

    SoundFormat           m_soundFormat;
    quint64               m_decodedSize;

    KUrl                  m_inputURL;

    QList<DataBuffer>     m_buffers;
    QSemaphore            m_bufferAccessLock;
    QSemaphore            m_bufferCountSemaphore;
};


#endif
